unit MainForm;

interface

{$A-}

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, MMSystem, mmErrMsg, ComCtrls,
  Stringhl;

const
  sample_rate = 8000;
  sine_table_samples = 1 shl 15;     // number of samples in sine table
  max_buffer_samples = 56000;        // reasonable size of output buffer (< 64K)
  open_error = 'Error opening waveform audio!';
  mem_error = 'Error allocating memory!';

type
  audio_sample = -32767..32767;       // for 16-bit audio

type
  PSineTable = ^TSineTable;          // sine value store
  TSineTable = array [0..sine_table_samples-1] of audio_sample;

  PBuffer = ^TBuffer;                // output buffer type
  TBuffer = array [0..max_buffer_samples-1] of audio_sample;

type
  TMorseMain = class(TForm)
    StartBtn: TButton;
    PauseBtn: TButton;
    StatusBar: TStatusBar;
    GroupBox1: TGroupBox;
    SpeedTrackBar: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    SpacingTrackBar: TTrackBar;
    TextWin: TMemo;
    TextType: TRadioGroup;
    GroupBox3: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    PitchTrackBar: TTrackBar;
    OpenDialog: TOpenDialog;
    QSOs: TStringHolder;
    Prefixes: TStringHolder;
    Antennas: TStringHolder;
    Rigs: TStringHolder;
    Names: TStringHolder;
    Weather: TStringHolder;
    Label7: TLabel;
    GroupBox4: TGroupBox;
    VolTrackBar: TTrackBar;
    Chat: TStringHolder;
    TextFileBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure StartBtnClick(Sender: TObject);
    procedure PauseBtnClick(Sender: TObject);
    procedure SpeedTrackBarChange(Sender: TObject);
    procedure SpacingTrackBarChange(Sender: TObject);
    procedure PitchTrackBarChange(Sender: TObject);
    procedure TextFileBtnClick(Sender: TObject);
    procedure VolTrackBarChange(Sender: TObject);
  private
    { Private declarations }
    angle: integer;          // current sine wave angle
    sine_table: PSineTable;  // sine-wave values are pre-stored in this array
    p_wave_hdr1: PWaveHdr;   // wave headers
    p_wave_hdr2: PWaveHdr;
    p_buffer1: PBuffer;      // output buffers
    p_buffer2: PBuffer;
    hWave_hdr1: HGlobal;
    hWave_hdr2: HGlobal;
    hBuffer1: HGlobal;
    hBuffer2: HGlobal;
    buffer_bytes: integer;   // max number of bytes in each output buffer
    chars_sent, chars_played: integer;
    f: extended;
    v: integer;
    hWave_out: HWaveOut;     // handle to wave out device
    pcm: TWaveFormatEx;      // wave format descriptor
    key_down: boolean;
    shutoff: boolean;
    closed: boolean;
    procedure MorseKeyUp;
    procedure MorseKeyDown;
    function TextToCode(const text: string): string;
    procedure GenerateCode;

    // call-backs from waveform out functions
    procedure mm_wom_Open (var Msg: TMessage);  message mm_wom_open;
    procedure mm_wom_Done (var Msg: TMessage);  message mm_wom_done;
    procedure mm_wom_Close (var Msg: TMessage);  message mm_wom_close;

    procedure fill_buffer_with_sinewave (bfr: PBuffer;  var index, samples: integer);
		procedure fill_buffer_with_silence (bfr: pBuffer;  var index, samples: integer);
		function fill_buffer_with_char (bfr: pBuffer; ch: char): integer;
    procedure write_next_buffer (header: PWaveHdr);
    procedure do_sine_table;

  public
    { Public declarations }
  end;

var
  MorseMain: TMorseMain;

implementation

uses Utils;

{$R *.DFM}
{$I code.inc}

type
	elements = (space, dot, dash);

var
	codestr, morsestr, sentstr, testfile: string;
  dotlength, spacing, codeptr: integer;

procedure TMorseMain.FormCreate(Sender: TObject);
begin
	GetFormPosFromRegistry(sRegKeyRoot,'WindowPos',MorseMain,false);
  // get the memory required for wave headers
  // this code is probably irrelevant in the Win32 environment
  hWave_hdr1 := GlobalAlloc (gHnd or gMem_Share, SizeOf (TWaveHdr));
  p_wave_hdr1 := pWaveHdr (GlobalLock (hWave_hdr1));
  hWave_hdr2 := GlobalAlloc (gHnd or gMem_Share, SizeOf (TWaveHdr));
  p_wave_hdr2 := pWaveHdr (GlobalLock (hWave_hdr2));

  // estimate of reasonable output buffer size
	buffer_bytes := 2 * max_buffer_samples;

  // get the memory required for output buffers
  hBuffer1 := GlobalAlloc (gHnd or gMem_Share, buffer_bytes);
  p_buffer1 := pBuffer (GlobalLock (hBuffer1));
  hBuffer2 := GlobalAlloc (gHnd or gMem_Share, buffer_bytes);
  p_buffer2 := pBuffer (GlobalLock (hBuffer2));

  hWave_out := 0;
  // get the memory for the sine-wave table and build it
  GetMem (sine_table, SizeOf (TSineTable));
  do_sine_table;

  // set other state variables
  shutoff := false;
  key_down := false;
  f := 1000;

  Randomize;

  SpeedTrackBar.Position := GetIntFromRegistry(sRegKeyRoot,'Speed',12);
  SpacingTrackBar.Position := GetIntFromRegistry(sRegKeyRoot,'Spacing',0);
  TextType.ItemIndex := GetIntFromRegistry(sRegKeyRoot,'Test',0);
  PitchTrackBar.Position := GetIntFromRegistry(sRegKeyRoot,'Pitch',8);
  VolTrackBar.Position := GetIntFromRegistry(sRegKeyRoot,'Volume',0);

	SpeedTrackBarChange(Sender);
	SpacingTrackBarChange(Sender);
	PitchTrackBarChange(Sender);
	VolTrackBarChange(Sender);
  codestr := '';
  codeptr := 0;
  morsestr := '';
  testfile := '';
end;


procedure TMorseMain.FormDestroy(Sender: TObject);
begin
	WriteFormPosToRegistry(sRegKeyRoot,'WindowPos',MorseMain,false);
  WriteIntToRegistry(sRegKeyRoot,'Speed',SpeedTrackBar.Position);
  WriteIntToRegistry(sRegKeyRoot,'Spacing',SpacingTrackBar.Position);
  WriteIntToRegistry(sRegKeyRoot,'Test',TextType.ItemIndex);
  WriteIntToRegistry(sRegKeyRoot,'Pitch',PitchTrackBar.Position);
  WriteIntToRegistry(sRegKeyRoot,'Volume',VolTrackBar.Position);
  shutoff := true;
  GlobalUnlock (hWave_hdr1);  GlobalFree (hWave_hdr1);
  GlobalUnlock (hBuffer1);  GlobalFree (hBuffer1);
  GlobalUnlock (hWave_hdr2);  GlobalFree (hWave_hdr2);
  GlobalUnlock (hBuffer2);  GlobalFree (hBuffer2);
  FreeMem (sine_table, SizeOf (TSineTable));
end;


procedure TMorseMain.MorseKeyUp;
begin
  // is a sweep running?  if so, stop it
  if key_down then
  begin
    shutoff := true;
    closed := false;
    repeat
      Application.ProcessMessages;
    until closed;
  end
end;


procedure TMorseMain.MorseKeyDown;
var
  open_status: MMRESULT;
begin
  if key_down then MorseKeyUp;

  angle := 0;
  // fill in the TWaveFormatEx structure with our wave details
  with pcm do
  begin
    wFormatTag := wave_Format_PCM;         // it's PCM data
    nChannels := 1;                        // mono
    nSamplesPerSec := sample_rate;         // set the 44.1KHz rate
    nAvgBytesPerSec := 2 * sample_rate;    // two bytes per sample
    nBlockAlign := 2;                      // for mono 16-bit audio
    wBitsPerSample := 16;                  // 16-bit audio
    cbSize := 0;
  end;

  shutoff := false;
  // try and open the wave device for our format of wave data
  open_status := waveOutOpen (@hWave_out, 0, @pcm, Handle, 0, callback_window);

  if open_status = 0 then
  begin
    // prepare to receive the WaveOutOpen message to actually start sending data
    key_down := true;
    closed := false;
  end
  else
  begin
    key_down := false;
    hWave_out := 0;
    // inform user of failure
    MessageDlg (open_error + #13#10 + translate_mm_error (open_status),
                mtWarning, [mbOK], 0);
  end;
end;

function TMorseMain.TextToCode(const text: string): string;
var
	s: string;
	i,c: longint;
begin
	Result := '';
  i := 1;
  s := Uppercase(text);
  while i <= Length(text) do
  begin
  	c := Ord(s[i]);
    if (c >= 32) and (c <= 90) then
    	Result := Result + ' '+code[c];
    Inc(i);
  end;
end;

procedure TMorseMain.GenerateCode;
var
	i,j,p,r: Integer;
  f: TextFile;
  s,c: string;
  QTHs: TStringList;

  function RandChar( min, max: char): char;
  var
  	m,r: Integer;
  begin
  	m := Ord(min);
    r := Ord(max) - m + 1;
    Result := Chr(Random(r)+m);
  end;

begin
	codestr := '';
	case TextType.ItemIndex of
  0:	begin
        for i := 1 to 12 do
        begin
        	for j := 1 to 5 do
          	codestr := codestr + RandChar('0','9');
          codestr := codestr + ' ';
        end;
  		end;
  1:	begin
        for i := 1 to 12 do
        begin
        	for j := 1 to 5 do
          	codestr := codestr + RandChar('A','Z');
          codestr := codestr + ' ';
        end;
  		end;
  2:	if Length(testfile) > 0 then
		  begin
				AssignFile(f,testfile);
        Reset(f);
        while not Eof(f) do
        begin
        	ReadLn(f,s);
        	codestr := codestr + s + #13#10;
        end;
        CloseFile(f);
  		end
      else
      	StatusBar.SimpleText := 'No test file selected!';
  3:	begin
  			QTHs := TStringList.Create;
  			codestr := QSOs.Strings[Random(QSOs.Strings.Count)];
        // substitute chat text
  			s := Chat.Strings[Random(Chat.Strings.Count)];
        repeat
        	p := Pos('\V',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
          end;
        until p = 0;
        // substitute call A
        s := Prefixes.Strings[Random(Prefixes.Strings.Count)];
     		c := Copy(s,1,Pos(' ',s)-1);
        if c[1] in ['W','K','N'] then
        begin
        	r := Random(5);
          if r > 0 then
          	Insert(Chr(64+r),s,2);
        end;
        if not (c[Length(c)] in ['0'..'9']) then
        	c := c + RandChar('0','9');
        c := c+RandChar('A','Z')+RandChar('A','Z');
        if Random(10) > 0 then c := c+ RandChar('A','Z');
        if Random(20) = 0 then c := c + '/P';
        repeat
        	p := Pos('\A',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(c,codestr,p);
          end;
        until p = 0;
        // substitute call B
        s := Prefixes.Strings[Random(Prefixes.Strings.Count)];
        p := Pos(' ',s)-1;
     		c := Copy(s,1,p);
        Delete(s,1,p);
        s := Trim(s);
        repeat
        	p := Pos(';',s);
          if p > 0 then
          begin
          	QTHs.Add(Copy(s,1,p-1));
            Delete(s,1,p);
          end;
        until p = 0;
        if c[1] in ['W','K','N'] then
        begin
        	r := Random(5);
          if r > 0 then
          	Insert(Chr(64+r),s,2);
        end;
        if not (c[Length(c)] in ['0'..'9']) then
        	c := c + RandChar('0','9');
        c := c+RandChar('A','Z')+RandChar('A','Z');
        if Random(10) > 0 then c := c+ RandChar('A','Z');
        if Random(20) = 0 then c := c + '/P' else if Random(40) = 0 then c := c + '/M';
        repeat
        	p := Pos('\B',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(c,codestr,p);
          end;
        until p = 0;
        // substitute call C
        s := Prefixes.Strings[Random(Prefixes.Strings.Count)];
        if c[1] in ['W','K','N'] then
        begin
        	r := Random(5);
          if r > 0 then
          	Insert(Chr(64+r),s,2);
        end;
     		c := Copy(s,1,Pos(' ',s)-1);
        if not (c[Length(c)] in ['0'..'9']) then
        	c := c + RandChar('0','9');
        c := c+RandChar('A','Z')+RandChar('A','Z');
        if Random(10) > 0 then c := c+ RandChar('A','Z');
        repeat
        	p := Pos('\C',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(c,codestr,p);
          end;
        until p = 0;
        r := 0;
        // substitute RST
        s := RandChar('3','5')+RandChar('1','9')+RandChar('6','9');
        repeat
        	p := Pos('\S',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
            Inc(r);
          end;
        until p = 0;
        // substitute power
        s := RandChar('1','9');
        for i := 0 to Random(2) do
        	s := s + RandChar('0','9');
        repeat
        	p := Pos('\P',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
          end;
        until p = 0;
        // substitute antenna
  			s := Antennas.Strings[Random(Antennas.Strings.Count)];
        repeat
        	p := Pos('\Z',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
          end;
        until p = 0;
        // substitute rig
  			s := Rigs.Strings[Random(Rigs.Strings.Count)];
        repeat
        	p := Pos('\R',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
          end;
        until p = 0;
        // substitute name
  			s := Names.Strings[Random(Names.Strings.Count)];
        repeat
        	p := Pos('\N',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
          end;
        until p = 0;
        // substitute qth
  			s := QTHs.Strings[Random(QTHs.Count)];
        if (Length(s) <= 12) and (r > 1) then s := s + ' ' + s;
        repeat
        	p := Pos('\Q',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
          end;
        until p = 0;
        // substitute weather
  			s := Weather.Strings[Random(Weather.Strings.Count)];
        repeat
        	p := Pos('\W',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
          end;
        until p = 0;
        // substitute greeting
        case Random(3) of
        0:	s := 'GM';
        1:	s := 'GE';
  			else
        	s := 'GD'
        end;
        repeat
        	p := Pos('\G',codestr);
          if p > 0 then
          begin
          	Delete(codestr,p,2);
            Insert(s,codestr,p);
          end;
        until p = 0;
        QTHs.Free;
  		end;
  end;
  if Length(codestr) > 0 then
  begin
  	codestr := 'VVV  VVV  '#13#10+codestr+' ';
	  codeptr := 1;
  end;
end;

procedure TMorseMain.mm_wom_open (var Msg: tMessage);
// This code handles the WaveOutOpen message by writing two buffers of data
// to the wave device.  Plus other miscellaneous housekeeping.
begin
  // populate the first wave header
  with p_wave_hdr1^ do
  begin
    lpData := pChar (p_buffer1);   // pointer to the data
    dwBufferLength := 0;           // fill in size later
    dwBytesRecorded := 0;
    dwUser := 0;
    dwFlags := 0;
    dwLoops := 1;                  // just a single loop
    lpNext := nil;
    reserved := 0;
  end;

  // populate the second wave header
  with p_wave_hdr2^ do
  begin
    lpData := pChar (p_buffer2);   // pointer to the data
    dwBufferLength := 0;           // fill in size later
    dwBytesRecorded := 0;
    dwUser := 0;
    dwFlags := 0;
    dwLoops := 1;                  // just a single loop
    lpNext := nil;
    reserved := 0;
  end;

//  chars_sent := 0;
//  chars_played := 0;
  p_wave_hdr1^.dwBufferLength := buffer_bytes;     // actual buffer sizes
  p_wave_hdr2^.dwBufferLength := buffer_bytes;
  // now write the first two buffers into the wave output
  waveOutPrepareHeader (hWave_out, p_wave_hdr1, SizeOf (TWaveHdr));
  write_next_buffer (p_wave_hdr1);
  waveOutPrepareHeader (hWave_out, p_wave_hdr2, SizeOf (TWaveHdr));
  write_next_buffer (p_wave_hdr2);
end;


procedure TMorseMain.write_next_buffer (header: pWaveHdr);
begin
  if shutoff then Exit;
  with header^ do
    dwBufferLength := fill_buffer_with_char (pBuffer (lpData), codestr[codeptr]);
  sentstr := sentstr+codestr[codeptr];
  inc(codeptr);
  // write the buffer and bump the number written
  waveOutWrite (hWave_out, header, SizeOf (TWaveHdr));
  Inc(chars_sent);
end;


procedure TMorseMain.mm_wom_done (var Msg: tMessage);
// handle the wave out done message by writing the next buffer, if required
var
   free_header: pWaveHdr;
begin
	Inc(chars_played);
  if sentstr[chars_played] = ' ' then
  begin
		TextWin.Lines.SetText(PChar(Copy(sentstr,1,chars_played)));
    TextWin.Perform(EM_SETSEL,chars_played-1,chars_played-1);
    TextWin.Perform(EM_SCROLLCARET,0,0);
  end;
  // point to wave header just completed, i.e. the next free buffer
  free_header := pWaveHdr (msg.lParam);
  if not shutoff then
  begin
    if codeptr > Length(codestr) then
    begin
      // everything written has been played
      shutoff := true;
      key_down := false;
    end
    else
    begin
      // and write the next buffer, re-using the one just played
      write_next_buffer (free_header);
    end
  end;
  if shutoff then
  begin
    waveOutReset (hWave_out);
    waveOutClose (hWave_out);
  end;
end;


procedure TMorseMain.mm_wom_close (var Msg: tMessage);
// handle the wave out close message, release the wave headers
begin
  waveOutUnprepareHeader (hWave_out, p_wave_hdr1, SizeOf (TWaveHdr));
  waveOutUnprepareHeader (hWave_out, p_wave_hdr2, SizeOf (TWaveHdr));
  p_wave_hdr1 := pWaveHdr (GlobalLock (hWave_hdr1));
  if p_wave_hdr1 = nil then
    ShowMessage ('Failed to re-lock buffer p_wave_hdr1!');
  p_wave_hdr2 := pWaveHdr (GlobalLock (hWave_hdr2));
  if p_wave_hdr2 = nil then
    ShowMessage ('Failed to re-lock buffer p_wave_hdr2!');
  hWave_out := 0;
  if codeptr > Length(codestr) then
  begin
	  StatusBar.SimpleText := 'Test finished';
  	PauseBtn.Enabled := false;
  end
  else
  begin
	  StatusBar.SimpleText := 'Test paused';
    PauseBtn.Caption := '&Resume';
  end;
  key_down := false;
  closed := true;
  StartBtn.Enabled := true;
end;


procedure TMorseMain.do_sine_table;
var
  i: 0..sine_table_samples - 1;
  y, vol: extended;
begin
	vol := v / 8.0;
  // yes, I realise we could symmetry to reduce the number of computations
  // required, but it really doesn't take that long.
  for i := 0 to sine_table_samples - 1 do
    begin
    // Assume 16-bit audio goes from -32767..32767, avoids clipping.
    // There are only 2^15 samples here, this simplfies the subsequent angle
    // calculation but might restrict the dynamic range produced with noise
    // sidebands.  However, in the quality of equipment likely to be
    // encountered this won't matter.  You've got the source code, so
    // you can alter this if you like.
    y := round (vol * 32767.0 * sin (2.0* i * Pi / sine_table_samples));
    sine_table^ [i] := round (y);
    end;
end;


procedure TMorseMain.fill_buffer_with_sinewave (bfr: pBuffer;  var index, samples: integer);
const
  fract_bits = 15;
var
  sample: audio_sample;
  d_angle: integer;      // 32-bit number, with 14 fractional bits, i.e. 17.15
  i,j,max_angle: integer;
begin
	angle := 0;
  // compute the angular step per sample corresponding to the desired frequency
  d_angle := round ((sine_table_samples shl fract_bits) * f / sample_rate);
  // this is the maximum number of samples in the sine table
  max_angle := (sine_table_samples shl fract_bits) - 1;
  for i := 1 to samples do
  begin
  	sample := sine_table^[angle shr fract_bits];// get current sine value
    if i < 32 then
      for j := i to 32 do
      	sample := (sample * 8) div 9;
    if i > (samples - 32) then
    	for j := samples-32 to i do
      	sample := (sample * 8) div 9;
    bfr^ [index] := sample;
    Inc (index);                               // bump the buffer pointer
    Inc (angle, d_angle);                      // bump the angle
    angle := angle and max_angle;              // wrap to 360 degrees
  end;
end;


procedure TMorseMain.fill_buffer_with_silence (bfr: pBuffer;  var index, samples: integer);
var
  sample: integer;
begin
  for sample := 0 to samples - 1 do
  begin
    bfr^ [index] := 0;                         // store it in the caller's buffer
    Inc (index);                               // bump the buffer pointer
  end;
end;


function TMorseMain.fill_buffer_with_char (bfr: pBuffer; ch: char): integer;
// This procedure fills a  buffer with a Morse character.
var
  sample, chunk_samples, c, i: integer;
  mc: string;
begin
  sample := 0;
  c := Ord(ch);
  if (c >= 32) and (c <= 90) then
  begin
  	mc := code[c];
    i := 1;
    while i <= Length(mc) do
    begin
    	case mc[i] of
      '.':	begin
      				chunk_samples := dotlength;
			      	fill_buffer_with_sinewave(bfr, sample, chunk_samples);
			      	fill_buffer_with_silence(bfr, sample, dotlength);
      			end;
      '-':	begin
      				chunk_samples := dotlength * 3;
			      	fill_buffer_with_sinewave(bfr, sample, chunk_samples);
			      	fill_buffer_with_silence(bfr, sample, dotlength);
      			end;
      else
      	chunk_samples := dotlength * 2;
      	fill_buffer_with_silence(bfr, sample, chunk_samples);
      end;
      Inc(i);
    end;
	end;
	chunk_samples := dotlength * (2+spacing);
	fill_buffer_with_silence(bfr, sample, chunk_samples);
  Result := sample * 2; // no of bytes (2 per sample)
end;


procedure TMorseMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  MorseKeyUp;
end;

procedure TMorseMain.StartBtnClick(Sender: TObject);
begin
	SpeedTrackBarChange(Sender);
	SpacingTrackBarChange(Sender);
	GenerateCode;
	morsestr := '';
  sentstr := '';
  TextWin.Lines.Clear;
  if Length(codestr) > 0 then
  begin
    StatusBar.SimpleText := 'Test started';
    PauseBtn.Caption := '&Pause';
    PauseBtn.Enabled := true;
    StartBtn.Enabled := false;
    chars_sent := 0;
    chars_played := 0;
    MorseKeyDown;
  end;
end;

procedure TMorseMain.PauseBtnClick(Sender: TObject);
begin
 	if key_down then
  	shutoff := true
  else
  	if codeptr <= Length(codestr) then
    begin
    	Dec(codeptr);
      Dec(chars_played);
      Delete(sentstr,Length(sentstr),1);
		  StatusBar.SimpleText := 'Test resumed';
      PauseBtn.Caption := '&Pause';
      StartBtn.Enabled := false;
      MorseKeyDown;
    end;
end;

procedure TMorseMain.SpeedTrackBarChange(Sender: TObject);
var
	s: Integer;
begin
	s := SpeedTrackBar.Position;
	dotlength := ((sample_rate * 12) div 10) div s;
	SpeedTrackBar.Hint := IntToStr(s)+' wpm';
  Caption := 'MorseGen - '+SpeedTrackBar.Hint+' '+SpacingTrackBar.Hint;
end;

procedure TMorseMain.SpacingTrackBarChange(Sender: TObject);
begin
	spacing := SpacingTrackBar.Position;
  case spacing of
  0: SpacingTrackBar.Hint := 'Normal spacing';
  1,2,3: SpacingTrackBar.Hint := 'Increased spacing';
  else
  	SpacingTrackBar.Hint := 'Long spacing';
  end;
  Caption := 'MorseGen - '+SpeedTrackBar.Hint+' '+SpacingTrackBar.Hint;
end;

procedure TMorseMain.PitchTrackBarChange(Sender: TObject);
begin
	f := PitchTrackBar.Position * 100;
	PitchTrackBar.Hint := IntToStr(Round(f))+' Hz';
end;

procedure TMorseMain.VolTrackBarChange(Sender: TObject);
begin
	v := 8 - VolTrackBar.Position;
	VolTrackBar.Hint := IntToStr(v);
  do_sine_table;
end;

procedure TMorseMain.TextFileBtnClick(Sender: TObject);
begin
	if TextType.ItemIndex = 2 then
  begin
  	if OpenDialog.Execute then
    	testfile := OpenDialog.FileName;
  end;
end;

end.
